
local addon  = KiwiPlates
local status = addon:newStatus( 'roster' )

local UnitGUID = UnitGUID
local UnitIsUnit = UnitIsUnit
local GetUnitName = GetUnitName
local GetNumGroupMembers = GetNumGroupMembers
local UnitGroupRolesAssigned = UnitGroupRolesAssigned

local arena_units = { 'arena1', 'arena2', 'arena3', 'arena4', 'arena5' }
local arena_pets  = { 'arenapet1', 'arenapet2', 'arenapet3', 'arenapet4', 'arenapet5' }
local group_units = { [true] = {}, [false] = { 'player', 'party1', 'party2', 'party3', 'party4' } }
for i=1,MAX_RAID_MEMBERS do table.insert( group_units[true], 'raid'..i ) end

local unitByGuid = {}
local unitByName = {}
local offtankByGuid = {}
local offtankByUnit = {}

local tables = {}
local function UpdateUnit(unit)
	local data = tables[unit]
	if not data then
		data = { unit = unit }
		tables[unit] = data
	end
	data.guid      = UnitGUID(unit)
	data.name      = GetUnitName(unit)
	data.healthMax = UnitHealthMax(unit)			
	_, data.class  = UnitClass(unit)
	unitByName[data.name] = data
	unitByGuid[data.guid] = data
end

local function Update()
	wipe(unitByGuid)
	wipe(unitByName)
	wipe(offtankByGuid)
	wipe(offtankByUnit)
	local m, _ = GetNumGroupMembers()
	if m>1 then
		local units = group_units[IsInRaid()]
		for i=1,m do
			local unit = units[i]
			UpdateUnit( unit )
			if UnitGroupRolesAssigned(unit) == 'TANK' and (not UnitIsUnit(unit,'player')) then
				local guid = UnitGUID(unit)
				offtankByGuid[ guid ] = unit
				offtankByUnit[ unit ] = guid
			end
		end
	end
end	

local function UpdateArena()
    for i = 1, GetNumArenaOpponents() do
		UpdateUnit( arena_units[i] )
		UpdateUnit( arena_pets[i] )
	end	
end

local function PlateShow(bPlate)
	local info = bPlate.guid and unitByGuid[bPlate.guid] or unitByName[bPlate.bNameText:GetText()]
	if info then
		bPlate.vHealthMax = info.healthMax
	end
end

function status:OnEnable(addon, dbx)
	Update()
	UpdateArena()
	addon:RegisterEvent('GROUP_ROSTER_UPDATE', Update)
	addon:RegisterEvent('PARTY_MEMBERS_CHANGED', Update)
	addon:RegisterEvent('ARENA_OPPONENT_UPDATE', UpdateArena)
	addon.RegisterNotify( 'bPlateShow' , PlateShow )	
end

function status:OnDisable(addon)
	wipe(unitByGuid)
	wipe(unitByName)
	wipe(offtankByGuid)
	wipe(offtankByUnit)
	addon:UnregisterEvent('GROUP_ROSTER_UPDATE')
	addon:UnregisterEvent('PARTY_MEMBERS_CHANGED')
	addon:UnregisterEvent('ARENA_OPPONENT_UPDATE')
	addon.UnregisterNotify( 'bPlateShow' , PlateShow )
end

local function GetPlateUnitInfo(bPlate)
	local guid = bPlate.guid
	if guid then 
		return unitByGuid[guid] 
	end
	return unitByName[bPlate.bNameText:GetText()]
end

addon.GetPlateUnitInfo = GetPlateUnitInfo
addon.unitByGuid    = unitByGuid
addon.unitByName    = unitByName
addon.offtankByGuid = offtankByGuid
addon.offtankByUnit = offtankByUnit

