
local addon = KiwiPlates

addon.PlateSkinRules = {}

----------------------------------------------------------------
-- Textures
----------------------------------------------------------------
do
	local textures = setmetatable( {}, { __index = function(t,k) local v={}; t[k]=v; return v end } )

	function addon:RegisterWidgetTexture( widgetName, key, filename)
		textures[widgetName][key] = filename
	end

	function addon:GetWidgetTexture(widgetName, key)
		return textures[widgetName][key]
	end
	
	function addon:GetWidgetTextures(widgetName)
		return textures[widgetName]
	end
	
	function addon:GetWidgetTexturesValues(widgetName)
		local values = {}
		for k,f in next,textures[widgetName] do
			values[f] = k
		end
		return values
	end
	
end

----------------------------------------------------------------
--
----------------------------------------------------------------

function addon:MakeSkinCheckFunction()
	self.PlateGetSkinName = self.CompileSkinCheckFunction(self.theme.Skins, self.PlateSkinRules )
end

function addon:UpdatePlateVisibility()
	if self.theme and self.theme.Plate then
		local enabled = self.theme.Plate.nameplateShowEnemies
		if enabled~=nil then SetCVar('nameplateShowEnemies', enabled and 1 or 0) end	
		local enabled = self.theme.Plate.nameplateShowFriends
		if enabled~=nil then SetCVar('nameplateShowFriends', enabled and 1 or 0) end
	end	
end

local instances = { none = 'default', arena = 'arena', pvp = 'pvp' }
function addon:GetThemeName()
	local role = GetSpecialization()
	role = role and GetSpecializationRole(role) or 'DAMAGER'
	if role == 'TANK' and self.PlayerClass == 'WARRIOR' and GetShapeshiftForm() ~= 2 then
		role = 'DAMAGER'
	end	
	self.PlayerRole = role
	local themes = self.db.general.themes[ role ]
	return themes[ instances[ select(2,IsInInstance()) or 'none' ] or 'instance' ] or themes.default or 'Basic'
end	


----------------------------------------------------------------
-- widgets & statuses
----------------------------------------------------------------
function addon:EnableWidgets(widgetsdb)
	if widgetsdb then
		for name, db in next,widgetsdb do
			if type(db)=='table' then
				local widget = self.widgets[name]
				if widget then
					widget:Enable(self.theme)
				end	
			end
		end
	end	
end

function addon:DisableWidgets()
	for _, widget in next, self.widgets do
		widget:Disable()
	end
end

function addon:DisableStatuses()
	for _, status in pairs(self.statuses) do
		if status.enabled then
			status.enabled = nil
			if status.OnDisable then 
				status:OnDisable(self) 
			end	
		end
	end
end

----------------------------------------------------------------
-- Skins
----------------------------------------------------------------

function addon:RegisterSkin( name, data )
	local skin = addon.skins[name]
	if skin then
		for k,v in next, data do
			if not skin[k] then	skin[k] = v	end
		end
	else
		data.name = name
		addon.skins[name] = data
		addon.backup.skins[name] = data
	end	
end

----------------------------------------------------------------
-- Themes
----------------------------------------------------------------

function addon:RegisterTheme( name, data )
	local theme = addon.themes[name]
	if theme then
		for k,v in next, data do
			if not theme[k] then theme[k] = v end
		end
	else
		data.name = name
		addon.themes[name] = data
		addon.backup.themes[name] = data
	end	
end

function addon:EnableTheme(theme)
	if type(theme) == 'string' then
		theme = self.themes[theme] or self.themes.Basic
	end
	if theme then
		self.theme = theme
		self:MakeSkinCheckFunction()
		self:EnableWidgets( theme )	
		for _,skin in next,theme.Skins do
			self:EnableWidgets( self.skins[ skin[1] ] )
		end
		self.UpdateStatuses()
		self.ReSkinPlates(true)	
	end	
end

function addon:DisableTheme()
	if self.theme then
		self:ResetAllPlateColors()
		self:DisableStatuses()
		self:DisableWidgets()
		self.theme = nil
	end	
end

do	
	local updateQueued	
	local C_Timer = C_Timer
	function addon:ThemeOnUpdate(event)
		if not updateQueued then
			updateQueued = true
			C_Timer.After(0.1,function() 
				self:ThemeUpdate()	
				updateQueued = nil
			end )
		end	
	end
end

function addon:ThemeUpdate(name)
	local force = (name == true)
	name = (not force) and name or self:GetThemeName() 
	if (not self.theme) or name ~= self.theme.name or force then
		self:DisableTheme()
		self:EnableTheme( name )
		self:UpdatePlateVisibility()
		collectgarbage()
	end	
end
	
function addon:InitTheme(event)
	self.PlayerClass = select(2,UnitClass('player'))
	self:ThemeUpdate()
	self:RegisterEvent( 'PLAYER_TALENT_UPDATE', 'ThemeOnUpdate' )
	self:RegisterEvent( 'PLAYER_SPECIALIZATION_CHANGED', 'ThemeOnUpdate' )
    if self.PlayerClass == 'WARRIOR' then
         self:RegisterEvent('UPDATE_SHAPESHIFT_FORM', 'ThemeOnUpdate')
	end
end
