local addon = KiwiPlates

local next    = next
local unpack  = unpack
local tremove = tremove
local tinsert = tinsert

local defaultColor = {1,1,1,1}
local defaultHighlightColor = { 1,1,1,0.15 }
local defaultCoord = {0,1,0,1}

-- misc constants
addon.defaultColor = defaultColor
addon.defaultCoord = defaultCoord

-- ipairs table delete by value
addon.tdelete = function(t,v)
	for i=#t,1,-1 do
		if t[i] == v then
			tremove( t, i)
			return v, i
		end
	end	
end

-- insert a value in a sorted ipairs table
addon.tsinsert = function(t,v)
	for i=#t,1,-1 do
		if value>t[i] then
			tinsert(t,i,value)
			return
		end
	end	
end

-- deep copy table
do
	local function CopyTable(src, dst)
		if type(dst)~="table" then dst = {} end
		for k,v in next,src do
			if type(v)=="table" then
				dst[k] = CopyTable(v,dst[k])
			elseif not dst[k] then
				dst[k] = v
			end
		end
		return dst
	end
	addon.CopyTable = CopyTable
end

-- addon.CreateHighlightTexture(frame, db)
local function CreateHighlightTexture(frame, skinName, db)
	local tex = frame[skinName] 
	if not tex then
		tex = frame:CreateTexture(nil, 'OVERLAY')
		frame[skinName] = tex
	end
	tex:SetBlendMode('ADD')
	if db then
		if db.filename then
			tex:SetTexture( db.filename )
			tex:SetVertexColor( unpack(db.highlightColor or defaultColor) )
		else
			tex:SetTexture( unpack(db.highlightColor or defaultHighlightColor) )
		end
		if db.coords then 
			tex:SetTexCoord( unpack(db.coords) ) 
		end
		if db.points then 
			tex:SetPoint('CENTER', db.points[1], db.points[2])
			tex:SetSize(db.points[3], db.points[4] )
		else	
			tex:SetAllPoints()				
		end
	else
		tex:SetTexture( unpack(defaultHighlightColor) )
		tex:SetAllPoints()
	end
	return tex
end
addon.CreateHighlightTexture = CreateHighlightTexture

-- addon.PlateSetHighlightTexture(kPlate, frame, dbname)
addon.PlateSetHighlightTexture = function( kPlate, frame, dbname )
	local name = kPlate.skinName
	local tex  = frame[name] or CreateHighlightTexture( frame, name, kPlate.db[dbname] )
	local old = frame.enabledTexture
	if old~=tex then
		if old then	old:Hide() end
		frame.enabledTexture = tex
		tex:Show()
	end
	frame:SetParent( kPlate )
	frame:SetFrameLevel( kPlate:GetFrameLevel() )	
	frame:SetAllPoints( kPlate )
	frame:Show()	
end

-- addon.CreatePlateTexture(kPlate, name, db)
addon.CreatePlateTexture = function(kPlate, container, name, db)
	local tex = container[name] or kPlate:CreateTexture()
	tex:SetDrawLayer( db.layer or 'BACKGROUND', db.sublayer or 0)
	tex:SetTexture( db.filename or 'Interface\\Addons\\KiwiPlates\\media\\glow' )
	tex.defaultColor = db.color or defaultColor
	tex:SetVertexColor( unpack(tex.defaultColor) )
	tex:SetTexCoord( unpack(db.coord or defaultCoord) )
	tex:SetPoint('CENTER', db.left or 0, db.top or 0 )
	tex:SetSize( db.width or kPlate.width, db.height or kPlate.height )
	tex.SetColor = db.acceptColors and tex.SetVertexColor or nil
	container[name] = tex
	return tex
end

-- addon.CreatePlateBorderTexture(kPlate, name, db)
addon.CreatePlateBorderTexture = function(kPlate, container, name, db)
	local tex = container[name] or kPlate:CreateTexture()
	local filename, mulX, mulY = db.filename
	if filename then
		mulX, mulY = db.mulX or 1.25 , db.mulY or 2
	else
		filename = 'Interface\\Addons\\KiwiPlates\\media\\glow'
		mulX, mulY = 1.25, 2
	end
	tex:SetDrawLayer( 'BACKGROUND', -5)
	tex:SetTexture( filename )
	tex.defaultColor = db.color or defaultColor
	tex:SetVertexColor( unpack(tex.defaultColor) )
	tex:SetTexCoord( unpack(db.coord or defaultCoord) )
	tex:SetPoint('CENTER', db.left or 0, db.top or 0 )
	tex:SetSize( (db.width or kPlate.width) * mulX, (db.height or kPlate.height) * mulY )
	tex.SetColor = db.acceptColors and tex.SetVertexColor or nil
	container[name] = tex
	return tex
end

-- addon.CreatePlateDecoration(kPlate, name, db)
addon.CreatePlateDecoration = function(kPlate, HealthBar, db)
	local barBSize  = (db.borderSize or 0) * 2	
	local x         = db.left or 0
	local y         = db.top or 0
	local width     = db.width  or kPlate.width
	local height    = db.height or kPlate.height		
	local barWidth  = width  - barBSize
	local barHeight = height - barBSize	
	local Background = HealthBar.Background or kPlate:CreateTexture()
	Background:SetDrawLayer( 'BACKGROUND', 0)
	Background:ClearAllPoints()
	Background:SetPoint('CENTER', x, y)
	Background:SetSize( barWidth, barHeight )
	Background:SetTexture( unpack(db.backColor) )
	HealthBar.Background = Background
	local Border = HealthBar.Border or kPlate:CreateTexture()
	Border:SetDrawLayer( 'BACKGROUND', -1)
	Border:ClearAllPoints()
	Border:SetPoint('CENTER', x, y)
	Border:SetSize( width, height )
	Border:SetTexture( unpack(db.borderColor) )
	HealthBar.Border = Border
	return Border, Background
end
