local media = LibStub("LibSharedMedia-3.0", true)

local addon = KiwiPlates
local Health = addon:newWidget( 'HealthBar', { 'health' } )

local unpack = unpack

local function Update(kPlate,bPlate)
	local HealthBar = kPlate.HealthBar
	if HealthBar then
		HealthBar:SetWidth( bPlate.vHealth * HealthBar.barWidth )
	end	
end

local function Create(kPlate)
	local db = kPlate.db.HealthBar 
	if db then
		local barBSize  = (db.borderSize or 0) * 2	
		local width     = db.width  or kPlate.width
		local height    = db.height or kPlate.height		
		local barWidth  = width  - barBSize
		local barHeight = height - barBSize	
		local HealthBar  = kPlate.HealthBar or kPlate:CreateTexture()
		HealthBar.kPlate = kPlate
		HealthBar:SetDrawLayer( 'BACKGROUND', 1)
		HealthBar:SetTexture( media:Fetch('statusbar', db.barTexture) )
		HealthBar.defaultColor = db.color or addon.defaultColor
		HealthBar:SetVertexColor( unpack(HealthBar.defaultColor) )
		HealthBar:ClearAllPoints()	
		HealthBar:SetPoint('LEFT',kPlate, 'CENTER', (db.left or 0)-barWidth/2, db.top or 0)
		HealthBar:SetSize( barWidth, barHeight )
		HealthBar.barWidth = barWidth
		HealthBar.SetColor = db.acceptColors and HealthBar.SetVertexColor or nil
		if db.Texture then 
			addon.CreatePlateTexture(kPlate, HealthBar, 'Texture', db.Texture) 
		elseif db.borderColor and db.backColor then
			local Border = addon.CreatePlateDecoration(kPlate, HealthBar, db)
			Border.defaultColor = db.borderColor
			Border.SetColor = db.acceptColors and Border.SetTexture or nil			
			kPlate.HealthBarBorder = Border
		end
		kPlate.HealthBar = HealthBar		
	end	
end

function Health:SetShown(kPlate, visible)
	local HealthBar = kPlate.HealthBar
	if HealthBar then 
		HealthBar:SetShown(visible) 
		if HealthBar.Texture    then HealthBar.Texture:SetShown(visible)    end
		if HealthBar.Border     then HealthBar.Border:SetShown(visible)     end
		if HealthBar.Background then HealthBar.Background:SetShown(visible) end
	end
end

function Health:GetColorReceivers(skin, receivers)
	local receivers = receivers or {}
	if skin.HealthBar.acceptColors then
		receivers.HealthBar = 'HealthBar'
		if skin.HealthBar.borderColor and skin.HealthBar.backColor then
			receivers.HealthBarBorder = 'HealthBar(Border)'
		end	
	end	
	return receivers
end

function Health:OnEnable(addon)
	addon.RegisterNotify( 'PlateCreate', Create )
	addon.RegisterNotify( 'PlateShow', Update )
	addon.RegisterNotify( 'PlateHealthUpdate', Update )
end

function Health:OnDisable(addon)
	addon.UnregisterNotify( 'PlateCreate', Create )
	addon.UnregisterNotify( 'PlateShow', Update )	
	addon.UnregisterNotify( 'PlateHealthUpdate', Update )
end

