
local addon   = KiwiPlates
local Options = KiwiPlatesOptions

local skin
local options = {}
local widgets = {}
local skins_values = {}
local widgets_list = {}

local orders = { Plate = 0, Target = 0.1, Mouseover = 0.2, GlowBorder = 0.3, Auras = -1, CastBar = -1 }

----------------------------------------------------------------
--
----------------------------------------------------------------

local optionEnabledWidgets = {
	type = 'multiselect',
	name = 'Enabled Widgets',
	order = 1,
	get = function(info, key) return type(widgets[key]) == 'table' end,
	set = function(info, key, value)
		if value then
			local bskin  = addon.backup.skins[skin.__SKIN_BASE or skin.name]
			skin[key]    = bskin and ( bskin[key] or (bskin.__DISABLED and bskin.__DISABLED[key]) ) or Options:GetWidgetDefaults(key, true)
			widgets[key] = Options:MakeWidgetOptions(key, skin[key] )
		else
			skin[key]    = nil
			widgets[key] = nil			
		end
		info.handler:OnChange(info)		
		addon.widgets[key]:OnSkinChange( skin )
	end,
	confirm = function(info, key, value)
		return (not value) and 'Are you sure you want to remove: '..key
	end,
	values = function()
		for name in next,Options.widget_defaults do
			if name ~= 'Plate' then	widgets_list[name] = name end	
		end
		return widgets_list
	end	
}

local function LoadSkin(info, value)
	skin = nil
	options.skin.name = ''
	wipe(widgets)
	if value then
		options.skin.name = value	
		skin = addon.skins[value]
		info.handler.skin = skin
		widgets.enabledWidgets = optionEnabledWidgets		
		for key,db in pairs(skin) do
			if type(db) == 'table' and strsub(key,1,2) ~= '__' and not db.__HIDDEN then
				widgets[key]       = Options:MakeWidgetOptions(key, db)
				widgets[key].order = orders[key] or widgets[key].order
			end	
		end
	end	
end

----------------------------------------------------------------
--
----------------------------------------------------------------

local handler = {}
function handler:OnChange(info)
	-- database modified, copy database if necessary
	local name = skin.name
	if not addon.db.skins[name] then
		addon.db.skins[name] = skin
		addon.backup.skins[name] = nil
	end
	if not skin.__MODIFIED then
		skin.__MODIFIED = true
	end	
	-- notify changes to widgets
	if self.dbname ~= 'Skins' then
		local widget = addon.widgets[self.dbname]
		if widget then
			widget:OnSkinChange(skin)
		end	
	end	
end

----------------------------------------------------------------
--
----------------------------------------------------------------

options.skins = {
	type = "select",
	order = 10,
	width = "normal",
	name = "Select Skin",
	desc = "",
	get = function(info, value)
		if not skin then
			local theme = Options:GetSelectedTheme() or addon.theme
			if theme then
				LoadSkin(info, theme.Skins[1][1] )
			end
		end
		return skin and skin.name
	end,
	set = LoadSkin,
	values = function()
		wipe(skins_values)
		for k,v in pairs(addon.skins) do
			skins_values[k] = k
		end
		return skins_values
	end
}

options.clone = {
	type = "input",
	order = 20,
	width = "normal",
	name = "Clone Skin",
	desc = "Type a name for the new skin and click Accept.",
	get = function() end,
	set = function(info, name) 
		skin = addon.CopyTable(skin)
		skin.__SKIN_BASE = skin.__SKIN_BASE or skin.name
		skin.name = name
		addon.db.skins[name] = skin
		addon.skins[name] = skin
		LoadSkin(info, name)
	end,
	hidden   = function() return not skin end,
	validate = function(info, name) return not addon.skins[name] and strlen(name)>3 or "Skin name too short or already in use, type another skin name"  end,
	confirm  = function() return "Are you sure you want to duplicate this skin ?" end,	
}

options.reset = {
	type = "execute",
	order = 30,
	width = "half",
	name = "Reset",
	func = function(info) 
		addon.db.skins[skin.name] = nil
		ReloadUI()
	end,
	hidden = function() return not (skin and skin.__MODIFIED and not skin.__SKIN_BASE) end,
	confirm = function() return "Are you sure you want to reset this skin ? UI will be Reloaded" end,
}

options.delete = {
	type = "execute",
	order = 40,
	width = "half",
	name = "Delete",
	func = function(info) 
		addon.skins[skin.name] = nil
		addon.db.skins[skin.name] = nil
		LoadSkin(info, nil)
	end,
	hidden   = function() return not (skin and skin.__SKIN_BASE) end,
	disabled = function() return addon:IsSkinEnabled(skin) end,
	confirm  = function() return "Are you sure you want to delete this skin ?"  end,
}

options.skin = {
	type = "group",
	name = "",
	args = widgets,
}

----------------------------------------------------------------
--
----------------------------------------------------------------

Options:RegisterCategoryOptions( 'Skins', function()
	skin = nil
	wipe(widgets)
	return options
end , { order = 30, handler = handler } )
